/////////////////////////////////////////////////////////////////////////////
// Implementation for a MFC class encapsulation of the synchronous sockets

/*
 * Copyright (c) 2003 - 2005 by Daniel Weber (daniel@astolat.de)
 * All rights reserved.
 *
 * This is a SSL/TLS-implementation for Windows-Sockets in C++ written by
 * Daniel Weber (daniel@astolat.de) based an examples provided by
 * Martin Ziacek (martin.ziacek@blueyonder.co.uk).
 *
 * You are allowed to use, modify and redistribute this source-code for
 * commercial and non-commercial software as long as the following
 * conditions are met:
 * - Copyright remains bei Daniel Weber and redistributions of the source-
 *   code must retain this copyright-notice, list of conditions and
 *   disclaimer.
 * - Redistributions in binary form must mention Daniel Weber as author
 *   of parts of the SSL/TLS-implementations at program startup and/or
 *   in the documentation.
 *
 * This source-code is provided by Daniel Weber "as is" and any express or
 * implied warranties, including but not limited to the implied warranties
 * of merchantability and fitness for a particular purpose are disclaimed.
 * The authors or contributors are not liable for any damages arising out
 * of the use of this source-code, even if advised of the possibility of
 * such damage.
 */

class CSyncSocketEx : public CObject
{
	DECLARE_DYNAMIC(CSyncSocketEx);

// Construction
public:
	CSyncSocketEx();
	CSyncSocketEx( const CSyncSocketEx& rSrc );
	virtual ~CSyncSocketEx();

	void operator=( const CSyncSocketEx& rSrc );
	BOOL Create( UINT nSocketPort = 0, int nSocketType = SOCK_STREAM, LPCTSTR lpszSocketAddress = NULL );

// Attributes
public:
	SOCKET m_hSocket;

	BOOL IsWritible( int nTimeout = 0 ) const;
	BOOL IsReadible( int nTimeout = 0 ) const;

	operator SOCKET() const;
	BOOL Attach( SOCKET hSocket );
	SOCKET Detach();

	BOOL GetPeerName( CString& rPeerAddress, UINT& rPeerPort );
	BOOL GetPeerName( SOCKADDR* lpSockAddr, int* lpSockAddrLen );

	BOOL GetSockName( CString& rSocketAddress, UINT& rSocketPort );
	BOOL GetSockName( SOCKADDR* lpSockAddr, int* lpSockAddrLen );

	BOOL SetSockOpt( int nOptionName, const void* lpOptionValue, int nOptionLen, int nLevel = SOL_SOCKET );
	BOOL GetSockOpt( int nOptionName, void* lpOptionValue, int* lpOptionLen, int nLevel = SOL_SOCKET );

	static int PASCAL GetLastError();

	int GetTimeout();
	void SetTimeout( int nTimeout );

protected:
	int m_nTimeout;

// Operations
public:
	virtual BOOL Accept( CSyncSocketEx& rConnectedSocket, SOCKADDR* lpSockAddr = NULL, int* lpSockAddrLen = NULL );

	BOOL Bind( UINT nSocketPort, LPCTSTR lpszSocketAddress = NULL );
	BOOL Bind( const SOCKADDR* lpSockAddr, int nSockAddrLen );

	virtual BOOL Close();

	virtual BOOL Connect( LPCTSTR lpszHostAddress, UINT nHostPort );
	virtual BOOL Connect( const SOCKADDR* lpSockAddr, int nSockAddrLen );

	BOOL IOCtl( long lCommand, DWORD* lpArgument );

	BOOL Listen( int nConnectionBacklog = 5 );

	virtual int Receive( void* lpBuf, int nBufLen, int nFlags = 0 );

	int ReceiveFrom( void* lpBuf, int nBufLen, CString& rSocketAddress, UINT& rSocketPort, int nFlags = 0 );
	int ReceiveFrom( void* lpBuf, int nBufLen, SOCKADDR* lpSockAddr, int* lpSockAddrLen, int nFlags = 0 );

	enum { receives = 0, sends = 1, both = 2 };
	BOOL ShutDown( int nHow = sends);

	virtual int Send( const void* lpBuf, int nBufLen, int nFlags = 0 );
	virtual int Send( LPCTSTR lpszBuf );

	int SendTo( const void* lpBuf, int nBufLen, UINT nHostPort, LPCTSTR lpszHostAddress = NULL, int nFlags = 0 );
	int SendTo( const void* lpBuf, int nBufLen, const SOCKADDR* lpSockAddr, int nSockAddrLen, int nFlags = 0 );

// Implementation
public:
	BOOL Socket( int nSocketType = SOCK_STREAM, int nProtocolType = 0, int nAddressFormat = PF_INET );

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
};

inline int CSyncSocketEx::Send( LPCTSTR lpszBuf )
{
	return Send( lpszBuf, _tcsclen( lpszBuf ) );
}

inline int PASCAL CSyncSocketEx::GetLastError()
{
	return ::WSAGetLastError();
}

inline int CSyncSocketEx::GetTimeout()
{
	return m_nTimeout;
}

inline void CSyncSocketEx::SetTimeout( int nTimeout )
{
	m_nTimeout = nTimeout;
}
